@@ -1,5 +1,5 @@
 //
-// Copyright (c) 2019-2021 Red Hat, Inc.
+// Copyright (c) 2019-2022 Red Hat, Inc.
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
 // You may obtain a copy of the License at
@@ -17,15 +17,12 @@ package controllers
 
 import (
 	"context"
+	"errors"
 	"fmt"
-	"net/http"
 	"strings"
 	"time"
 
 	devfilevalidation "github.com/devfile/api/v2/pkg/validation"
-	"github.com/devfile/devworkspace-operator/pkg/provision/sync"
-
-	"github.com/devfile/devworkspace-operator/apis/controller/v1alpha1"
 	controllerv1alpha1 "github.com/devfile/devworkspace-operator/apis/controller/v1alpha1"
 	"github.com/devfile/devworkspace-operator/controllers/workspace/metrics"
 	"github.com/devfile/devworkspace-operator/pkg/common"
@@ -34,14 +31,15 @@ import (
 	"github.com/devfile/devworkspace-operator/pkg/constants"
 	"github.com/devfile/devworkspace-operator/pkg/library/annotate"
 	containerlib "github.com/devfile/devworkspace-operator/pkg/library/container"
+	"github.com/devfile/devworkspace-operator/pkg/library/env"
 	"github.com/devfile/devworkspace-operator/pkg/library/flatten"
-	registry "github.com/devfile/devworkspace-operator/pkg/library/flatten/internal_registry"
 	"github.com/devfile/devworkspace-operator/pkg/library/projects"
+	"github.com/devfile/devworkspace-operator/pkg/provision/automount"
 	"github.com/devfile/devworkspace-operator/pkg/provision/metadata"
 	"github.com/devfile/devworkspace-operator/pkg/provision/storage"
+	"github.com/devfile/devworkspace-operator/pkg/provision/sync"
 	wsprovision "github.com/devfile/devworkspace-operator/pkg/provision/workspace"
 	"github.com/devfile/devworkspace-operator/pkg/timing"
-
 	"github.com/go-logr/logr"
 	"github.com/google/uuid"
 	coputil "github.com/redhat-cop/operator-utils/pkg/util"
@@ -70,8 +68,9 @@ const (
 // DevWorkspaceReconciler reconciles a DevWorkspace object
 type DevWorkspaceReconciler struct {
 	client.Client
-	Log    logr.Logger
-	Scheme *runtime.Scheme
+	NonCachingClient client.Client
+	Log              logr.Logger
+	Scheme           *runtime.Scheme
 }
 
 /////// CRD-related RBAC roles
@@ -84,9 +83,11 @@ type DevWorkspaceReconciler struct {
 // +kubebuilder:rbac:groups="",resources=namespaces;events,verbs=get;list;watch
 // +kubebuilder:rbac:groups="batch",resources=jobs,verbs=get;create;list;watch;update;patch;delete
 // +kubebuilder:rbac:groups=admissionregistration.k8s.io,resources=mutatingwebhookconfigurations;validatingwebhookconfigurations,verbs=get;list;watch;create;update;patch;delete
+// +kubebuilder:rbac:groups=authorization.k8s.io,resources=subjectaccessreviews;localsubjectaccessreviews,verbs=create
 // +kubebuilder:rbac:groups=rbac.authorization.k8s.io,resources=roles;rolebindings;clusterroles;clusterrolebindings,verbs=get;list;watch;create;update
 // +kubebuilder:rbac:groups=oauth.openshift.io,resources=oauthclients,verbs=get;list;watch;create;update;patch;delete;deletecollection
 // +kubebuilder:rbac:groups=monitoring.coreos.com,resources=servicemonitors,verbs=get;create
+// +kubebuilder:rbac:groups=config.openshift.io,resources=proxies,verbs=get,resourceNames=cluster
 // +kubebuilder:rbac:groups=apps,resourceNames=devworkspace-controller,resources=deployments/finalizers,verbs=update
 /////// Required permissions for workspace ServiceAccount
 // +kubebuilder:rbac:groups="",resources=pods/exec,verbs=create
@@ -94,14 +95,16 @@ type DevWorkspaceReconciler struct {
 // +kubebuilder:rbac:groups=apps;extensions,resources=deployments,verbs=get;list;watch
 // +kubebuilder:rbac:groups="",resources=secrets,resourceNames=workspace-credentials-secret,verbs=get;create;patch;delete
 // +kubebuilder:rbac:groups="",resources=configmaps,resourceNames=workspace-preferences-configmap,verbs=get;create;patch;delete
+// +kubebuilder:rbac:groups="metrics.k8s.io",resources=pods,verbs=get;list;watch
 
 func (r *DevWorkspaceReconciler) Reconcile(ctx context.Context, req ctrl.Request) (reconcileResult ctrl.Result, err error) {
 	reqLogger := r.Log.WithValues("Request.Namespace", req.Namespace, "Request.Name", req.Name)
 	clusterAPI := sync.ClusterAPI{
-		Client: r.Client,
-		Scheme: r.Scheme,
-		Logger: reqLogger,
-		Ctx:    ctx,
+		Client:           r.Client,
+		NonCachingClient: r.NonCachingClient,
+		Scheme:           r.Scheme,
+		Logger:           reqLogger,
+		Ctx:              ctx,
 	}
 
 	// Fetch the Workspace instance
@@ -163,8 +166,9 @@ func (r *DevWorkspaceReconciler) Reconcile(ctx context.Context, req ctrl.Request
 	// Handle stopped workspaces
 	if !workspace.Spec.Started {
 		timing.ClearAnnotations(workspace)
+		r.removeStartedAtFromCluster(ctx, workspace, reqLogger)
 		r.syncTimingToCluster(ctx, workspace, map[string]string{}, reqLogger)
-		return r.stopWorkspace(workspace, reqLogger)
+		return r.stopWorkspace(ctx, workspace, reqLogger)
 	}
 
 	// If this is the first reconcile for a starting workspace, mark it as starting now. This is done outside the regular
@@ -197,6 +201,7 @@ func (r *DevWorkspaceReconciler) Reconcile(ctx context.Context, req ctrl.Request
 
 	defer func() (reconcile.Result, error) {
 		r.syncTimingToCluster(ctx, clusterWorkspace, timingInfo, reqLogger)
+
 		// Don't accidentally suppress errors by overwriting here; only check for timeout when no error
 		// encountered in main reconcile loop.
 		if err == nil {
@@ -204,6 +209,11 @@ func (r *DevWorkspaceReconciler) Reconcile(ctx context.Context, req ctrl.Request
 				reconcileResult, err = r.failWorkspace(workspace, timeoutErr.Error(), metrics.ReasonInfrastructureFailure, reqLogger, &reconcileStatus)
 			}
 		}
+		if reconcileStatus.phase == dw.DevWorkspaceStatusRunning {
+			metrics.WorkspaceRunning(clusterWorkspace, reqLogger)
+			r.syncStartedAtToCluster(ctx, clusterWorkspace, reqLogger)
+		}
+
 		return r.updateWorkspaceStatus(clusterWorkspace, reqLogger, &reconcileStatus, reconcileResult, err)
 	}()
 
@@ -221,14 +231,13 @@ func (r *DevWorkspaceReconciler) Reconcile(ctx context.Context, req ctrl.Request
 	}
 
 	timing.SetTime(timingInfo, timing.ComponentsCreated)
-	// TODO#185 : Temporarily do devfile flattening in main reconcile loop; this should be moved to a subcontroller.
 	flattenHelpers := flatten.ResolverTools{
 		WorkspaceNamespace: workspace.Namespace,
 		Context:            ctx,
 		K8sClient:          r.Client,
-		InternalRegistry:   &registry.InternalRegistryImpl{},
-		HttpClient:         http.DefaultClient,
+		HttpClient:         httpClient,
 	}
+
 	flattenedWorkspace, warnings, err := flatten.ResolveDevWorkspace(&workspace.Spec.Template, flattenHelpers)
 	if err != nil {
 		return r.failWorkspace(workspace, fmt.Sprintf("Error processing devfile: %s", err), metrics.ReasonBadRequest, reqLogger, &reconcileStatus)
@@ -257,21 +266,40 @@ func (r *DevWorkspaceReconciler) Reconcile(ctx context.Context, req ctrl.Request
 
 	// Set finalizer on DevWorkspace if necessary
 	// Note: we need to check the flattened workspace to see if a finalizer is needed, as plugins could require storage
-	if isFinalizerNecessary(workspace, storageProvisioner) {
+	if storageProvisioner.NeedsStorage(&workspace.Spec.Template) {
 		coputil.AddFinalizer(clusterWorkspace, storageCleanupFinalizer)
 		if err := r.Update(ctx, clusterWorkspace); err != nil {
 			return reconcile.Result{}, err
 		}
 	}
 
-	// Add init container to clone projects
-	projects.AddProjectClonerComponent(&workspace.Spec.Template)
-
 	devfilePodAdditions, err := containerlib.GetKubeContainersFromDevfile(&workspace.Spec.Template)
 	if err != nil {
 		return r.failWorkspace(workspace, fmt.Sprintf("Error processing devfile: %s", err), metrics.ReasonBadRequest, reqLogger, &reconcileStatus)
 	}
 
+	// Add common environment variables and env vars defined via workspaceEnv attribute
+	if err := env.AddCommonEnvironmentVariables(devfilePodAdditions, clusterWorkspace, &workspace.Spec.Template); err != nil {
+		return r.failWorkspace(workspace, fmt.Sprintf("Failed to process workspace environment variables: %s", err), metrics.ReasonBadRequest, reqLogger, &reconcileStatus)
+	}
+
+	// Add init container to clone projects
+	if projectClone, err := projects.GetProjectCloneInitContainer(&workspace.Spec.Template); err != nil {
+		return r.failWorkspace(workspace, fmt.Sprintf("Failed to set up project-clone init container: %s", err), metrics.ReasonInfrastructureFailure, reqLogger, &reconcileStatus)
+	} else if projectClone != nil {
+		devfilePodAdditions.InitContainers = append(devfilePodAdditions.InitContainers, *projectClone)
+	}
+
+	// Add automount resources into devfile containers
+	if err := automount.ProvisionAutoMountResourcesInto(devfilePodAdditions, clusterAPI, workspace.Namespace); err != nil {
+		var fatalErr *automount.FatalError
+		if errors.As(err, &fatalErr) {
+			return r.failWorkspace(workspace, fmt.Sprintf("Failed to process automount resources: %s", err), metrics.ReasonBadRequest, reqLogger, &reconcileStatus)
+		} else {
+			return reconcile.Result{}, err
+		}
+	}
+
 	err = storageProvisioner.ProvisionStorage(devfilePodAdditions, workspace, clusterAPI)
 	if err != nil {
 		switch storageErr := err.(type) {
@@ -354,7 +382,9 @@ func (r *DevWorkspaceReconciler) Reconcile(ctx context.Context, req ctrl.Request
 	}
 	serviceAcctStatus := wsprovision.SyncServiceAccount(workspace, saAnnotations, clusterAPI)
 	if !serviceAcctStatus.Continue {
-		// FailStartup is not possible for generating the serviceaccount
+		if serviceAcctStatus.FailStartup {
+			return r.failWorkspace(workspace, serviceAcctStatus.Message, metrics.ReasonBadRequest, reqLogger, &reconcileStatus)
+		}
 		reqLogger.Info("Waiting for workspace ServiceAccount")
 		reconcileStatus.setConditionFalse(dw.DevWorkspaceServiceAccountReady, "Waiting for DevWorkspace ServiceAccount")
 		if !serviceAcctStatus.Requeue && serviceAcctStatus.Err == nil {
@@ -362,6 +392,13 @@ func (r *DevWorkspaceReconciler) Reconcile(ctx context.Context, req ctrl.Request
 		}
 		return reconcile.Result{Requeue: serviceAcctStatus.Requeue}, serviceAcctStatus.Err
 	}
+	if wsprovision.NeedsServiceAccountFinalizer(&workspace.Spec.Template) {
+		coputil.AddFinalizer(clusterWorkspace, serviceAccountCleanupFinalizer)
+		if err := r.Update(ctx, clusterWorkspace); err != nil {
+			return reconcile.Result{}, err
+		}
+	}
+
 	serviceAcctName := serviceAcctStatus.ServiceAccountName
 	reconcileStatus.setConditionTrue(dw.DevWorkspaceServiceAccountReady, "DevWorkspace serviceaccount ready")
 
@@ -409,7 +446,7 @@ func (r *DevWorkspaceReconciler) Reconcile(ctx context.Context, req ctrl.Request
 	return reconcile.Result{}, nil
 }
 
-func (r *DevWorkspaceReconciler) stopWorkspace(workspace *dw.DevWorkspace, logger logr.Logger) (reconcile.Result, error) {
+func (r *DevWorkspaceReconciler) stopWorkspace(ctx context.Context, workspace *dw.DevWorkspace, logger logr.Logger) (reconcile.Result, error) {
 	status := currentStatus{phase: dw.DevWorkspaceStatusStopping}
 	if workspace.Status.Phase == devworkspacePhaseFailing || workspace.Status.Phase == dw.DevWorkspaceStatusFailed {
 		status.phase = workspace.Status.Phase
@@ -419,7 +456,7 @@ func (r *DevWorkspaceReconciler) stopWorkspace(workspace *dw.DevWorkspace, logge
 		}
 	}
 
-	stopped, err := r.doStop(workspace, logger)
+	stopped, err := r.doStop(ctx, workspace, logger)
 	if err != nil {
 		return reconcile.Result{}, err
 	}
@@ -437,13 +474,13 @@ func (r *DevWorkspaceReconciler) stopWorkspace(workspace *dw.DevWorkspace, logge
 	return r.updateWorkspaceStatus(workspace, logger, &status, reconcile.Result{}, nil)
 }
 
-func (r *DevWorkspaceReconciler) doStop(workspace *dw.DevWorkspace, logger logr.Logger) (stopped bool, err error) {
+func (r *DevWorkspaceReconciler) doStop(ctx context.Context, workspace *dw.DevWorkspace, logger logr.Logger) (stopped bool, err error) {
 	workspaceDeployment := &appsv1.Deployment{}
 	namespaceName := types.NamespacedName{
 		Name:      common.DeploymentName(workspace.Status.DevWorkspaceId),
 		Namespace: workspace.Namespace,
 	}
-	err = r.Get(context.TODO(), namespaceName, workspaceDeployment)
+	err = r.Get(ctx, namespaceName, workspaceDeployment)
 	if err != nil {
 		if k8sErrors.IsNotFound(err) {
 			return true, nil
@@ -452,19 +489,19 @@ func (r *DevWorkspaceReconciler) doStop(workspace *dw.DevWorkspace, logger logr.
 	}
 
 	// Update DevWorkspaceRouting to have `devworkspace-started` annotation "false"
-	routing := &v1alpha1.DevWorkspaceRouting{}
+	routing := &controllerv1alpha1.DevWorkspaceRouting{}
 	routingRef := types.NamespacedName{
 		Name:      common.DevWorkspaceRoutingName(workspace.Status.DevWorkspaceId),
 		Namespace: workspace.Namespace,
 	}
-	err = r.Get(context.TODO(), routingRef, routing)
+	err = r.Get(ctx, routingRef, routing)
 	if err != nil {
 		if !k8sErrors.IsNotFound(err) {
 			return false, err
 		}
 	} else if routing.Annotations != nil && routing.Annotations[constants.DevWorkspaceStartedStatusAnnotation] != "false" {
 		routing.Annotations[constants.DevWorkspaceStartedStatusAnnotation] = "false"
-		err := r.Update(context.TODO(), routing)
+		err := r.Update(ctx, routing)
 		if err != nil {
 			if k8sErrors.IsConflict(err) {
 				return false, nil
@@ -473,20 +510,24 @@ func (r *DevWorkspaceReconciler) doStop(workspace *dw.DevWorkspace, logger logr.
 		}
 	}
 
-	replicas := workspaceDeployment.Spec.Replicas
-	if replicas == nil || *replicas > 0 {
-		logger.Info("Stopping workspace")
-		err = wsprovision.ScaleDeploymentToZero(workspace, r.Client)
-		if err != nil && !k8sErrors.IsConflict(err) {
-			return false, err
+	// CleanupOnStop should never be nil as a default is always set
+	if config.Workspace.CleanupOnStop == nil || !*config.Workspace.CleanupOnStop {
+		replicas := workspaceDeployment.Spec.Replicas
+		if replicas == nil || *replicas > 0 {
+			logger.Info("Stopping workspace")
+			err = wsprovision.ScaleDeploymentToZero(ctx, workspace, r.Client)
+			if err != nil && !k8sErrors.IsConflict(err) {
+				return false, err
+			}
+			return false, nil
 		}
-		return false, nil
-	}
 
-	if workspaceDeployment.Status.Replicas == 0 {
-		return true, nil
+		return workspaceDeployment.Status.Replicas == 0, nil
+	} else {
+		logger.Info("Cleaning up workspace-owned objects")
+		requeue, err := r.deleteWorkspaceOwnedObjects(ctx, workspace)
+		return !requeue, err
 	}
-	return false, nil
 }
 
 // failWorkspace marks a workspace as failed by setting relevant fields in the status struct.
@@ -523,6 +564,42 @@ func (r *DevWorkspaceReconciler) syncTimingToCluster(
 	}
 }
 
+func (r *DevWorkspaceReconciler) syncStartedAtToCluster(
+	ctx context.Context, workspace *dw.DevWorkspace, reqLogger logr.Logger) {
+
+	if workspace.Annotations == nil {
+		workspace.Annotations = map[string]string{}
+	}
+
+	if _, hasStartedAtAnnotation := workspace.Annotations[constants.DevWorkspaceStartedAtAnnotation]; hasStartedAtAnnotation {
+		return
+	}
+
+	workspace.Annotations[constants.DevWorkspaceStartedAtAnnotation] = timing.CurrentTime()
+	if err := r.Update(ctx, workspace); err != nil {
+		if k8sErrors.IsConflict(err) {
+			reqLogger.Info("Got conflict when trying to apply started-at annotations to workspace")
+		} else {
+			reqLogger.Error(err, "Error trying to apply started-at annotation to devworkspace")
+		}
+	}
+}
+
+func (r *DevWorkspaceReconciler) removeStartedAtFromCluster(
+	ctx context.Context, workspace *dw.DevWorkspace, reqLogger logr.Logger) {
+	if workspace.Annotations == nil {
+		workspace.Annotations = map[string]string{}
+	}
+	delete(workspace.Annotations, constants.DevWorkspaceStartedAtAnnotation)
+	if err := r.Update(ctx, workspace); err != nil {
+		if k8sErrors.IsConflict(err) {
+			reqLogger.Info("Got conflict when trying to apply timing annotations to workspace")
+		} else {
+			reqLogger.Error(err, "Error trying to apply timing annotations to devworkspace")
+		}
+	}
+}
+
 func getWorkspaceId(instance *dw.DevWorkspace) (string, error) {
 	uid, err := uuid.Parse(string(instance.UID))
 	if err != nil {
@@ -557,6 +634,8 @@ func dwRelatedPodsHandler() handler.EventHandler {
 }
 
 func (r *DevWorkspaceReconciler) SetupWithManager(mgr ctrl.Manager) error {
+	setupHttpClients()
+
 	maxConcurrentReconciles, err := config.GetMaxConcurrentReconciles()
 	if err != nil {
 		return err
