@@ -1,13 +1,16 @@
 //
-// Copyright (c) 2019-2021 Red Hat, Inc.
-// This program and the accompanying materials are made
-// available under the terms of the Eclipse Public License 2.0
-// which is available at https://www.eclipse.org/legal/epl-2.0/
+// Copyright (c) 2019-2022 Red Hat, Inc.
+// Licensed under the Apache License, Version 2.0 (the "License");
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
 //
-// SPDX-License-Identifier: EPL-2.0
+//     http://www.apache.org/licenses/LICENSE-2.0
 //
-// Contributors:
-//   Red Hat, Inc. - initial API and implementation
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an "AS IS" BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
 //
 
 package main
@@ -19,10 +22,9 @@ import (
 	"os"
 	"runtime"
 
-	corev1 "k8s.io/api/core/v1"
-
 	"github.com/devfile/devworkspace-operator/controllers/controller/devworkspacerouting"
 	"github.com/devfile/devworkspace-operator/controllers/controller/devworkspacerouting/solvers"
+	"github.com/devfile/devworkspace-operator/pkg/cache"
 	"github.com/devfile/devworkspace-operator/pkg/config"
 	"github.com/devfile/devworkspace-operator/pkg/infrastructure"
 	"github.com/devfile/devworkspace-operator/pkg/webhook"
@@ -34,14 +36,18 @@ import (
 	controllerv1alpha1 "github.com/devfile/devworkspace-operator/apis/controller/v1alpha1"
 	workspacecontroller "github.com/devfile/devworkspace-operator/controllers/workspace"
 
+	configv1 "github.com/openshift/api/config/v1"
 	oauthv1 "github.com/openshift/api/oauth/v1"
 	routev1 "github.com/openshift/api/route/v1"
+	securityv1 "github.com/openshift/api/security/v1"
 	templatev1 "github.com/openshift/api/template/v1"
+	corev1 "k8s.io/api/core/v1"
 	k8sruntime "k8s.io/apimachinery/pkg/runtime"
 	utilruntime "k8s.io/apimachinery/pkg/util/runtime"
 	clientgoscheme "k8s.io/client-go/kubernetes/scheme"
 	_ "k8s.io/client-go/plugin/pkg/client/auth/gcp"
 	ctrl "sigs.k8s.io/controller-runtime"
+	"sigs.k8s.io/controller-runtime/pkg/client"
 	ctrlconfig "sigs.k8s.io/controller-runtime/pkg/client/config"
 	"sigs.k8s.io/controller-runtime/pkg/healthz"
 	"sigs.k8s.io/controller-runtime/pkg/log/zap"
@@ -71,6 +77,11 @@ func init() {
 		utilruntime.Must(routev1.Install(scheme))
 		utilruntime.Must(templatev1.Install(scheme))
 		utilruntime.Must(oauthv1.Install(scheme))
+		// Enable controller to manage SCCs in OpenShift; permissions to do this are not requested
+		// by default and must be added by a cluster-admin.
+		utilruntime.Must(securityv1.Install(scheme))
+		// Enable controller to read cluster-wide proxy on OpenShift
+		utilruntime.Must(configv1.AddToScheme(scheme))
 	}
 
 	// +kubebuilder:scaffold:scheme
@@ -94,6 +105,12 @@ func main() {
 	setupLog.Info(fmt.Sprintf("Commit: %s", version.Commit))
 	setupLog.Info(fmt.Sprintf("BuildTime: %s", version.BuildTime))
 
+	cacheFunc, err := cache.GetCacheFunc()
+	if err != nil {
+		setupLog.Error(err, "failed to set up objects cache")
+		os.Exit(1)
+	}
+
 	mgr, err := ctrl.NewManager(ctrl.GetConfigOrDie(), ctrl.Options{
 		Scheme:                 scheme,
 		MetricsBindAddress:     metricsAddr,
@@ -101,6 +118,7 @@ func main() {
 		HealthProbeBindAddress: ":6789",
 		LeaderElection:         enableLeaderElection,
 		LeaderElectionID:       "8d217f93.devfile.io",
+		NewCache:               cacheFunc,
 	})
 	if err != nil {
 		setupLog.Error(err, "unable to start manager")
@@ -111,9 +129,15 @@ func main() {
 		os.Exit(1)
 	}
 
+	nonCachingClient, err := client.New(mgr.GetConfig(), client.Options{Scheme: scheme})
+	if err != nil {
+		setupLog.Error(err, "unable to initialize non-caching client")
+		os.Exit(1)
+	}
+
 	// Index Events on involvedObject.name to allow us to get events involving a DevWorkspace's pod(s). This is used to
 	// check for issues that prevent the pod from starting, so that DevWorkspaces aren't just hanging indefinitely.
-	if err := mgr.GetFieldIndexer().IndexField(context.Background(), &corev1.Event{}, "involvedObject.name", func(obj k8sruntime.Object) []string {
+	if err := mgr.GetFieldIndexer().IndexField(context.Background(), &corev1.Event{}, "involvedObject.name", func(obj client.Object) []string {
 		ev := obj.(*corev1.Event)
 		return []string{ev.InvolvedObject.Name}
 	}); err != nil {
@@ -126,14 +150,16 @@ func main() {
 		Log:          ctrl.Log.WithName("controllers").WithName("DevWorkspaceRouting"),
 		Scheme:       mgr.GetScheme(),
 		SolverGetter: &solvers.SolverGetter{},
+		DebugLogging: config.ExperimentalFeaturesEnabled(),
 	}).SetupWithManager(mgr); err != nil {
 		setupLog.Error(err, "unable to create controller", "controller", "DevWorkspaceRouting")
 		os.Exit(1)
 	}
 	if err = (&workspacecontroller.DevWorkspaceReconciler{
-		Client: mgr.GetClient(),
-		Log:    ctrl.Log.WithName("controllers").WithName("DevWorkspace"),
-		Scheme: mgr.GetScheme(),
+		Client:           mgr.GetClient(),
+		NonCachingClient: nonCachingClient,
+		Log:              ctrl.Log.WithName("controllers").WithName("DevWorkspace"),
+		Scheme:           mgr.GetScheme(),
 	}).SetupWithManager(mgr); err != nil {
 		setupLog.Error(err, "unable to create controller", "controller", "DevWorkspace")
 		os.Exit(1)
@@ -180,21 +206,14 @@ func main() {
 }
 
 func setupControllerConfig(mgr ctrl.Manager) error {
-	operatorNamespace, err := infrastructure.GetOperatorNamespace()
-	if err == nil {
-		config.ConfigMapReference.Namespace = operatorNamespace
-	} else {
-		config.ConfigMapReference.Namespace = os.Getenv(infrastructure.WatchNamespaceEnvVar)
-	}
-	err = config.WatchControllerConfig(mgr)
+	nonCachedClient, err := client.New(mgr.GetConfig(), client.Options{
+		Scheme: mgr.GetScheme(),
+	})
 	if err != nil {
 		return err
 	}
-
-	err = config.ControllerCfg.Validate()
-	if err != nil {
-		setupLog.Error(err, "Controller configuration is invalid")
+	if err := config.MigrateConfigFromConfigMap(nonCachedClient); err != nil {
 		return err
 	}
-	return nil
+	return config.SetupControllerConfig(nonCachedClient)
 }
