@@ -1,5 +1,5 @@
 //
-// Copyright (c) 2019-2021 Red Hat, Inc.
+// Copyright (c) 2019-2022 Red Hat, Inc.
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
 // You may obtain a copy of the License at
@@ -19,6 +19,9 @@ import (
 	"fmt"
 	"regexp"
 	"strings"
+
+	dw "github.com/devfile/api/v2/pkg/apis/workspaces/v1alpha2"
+	"github.com/devfile/devworkspace-operator/pkg/constants"
 )
 
 var NonAlphaNumRegexp = regexp.MustCompile(`[^a-z0-9]+`)
@@ -34,6 +37,13 @@ func EndpointName(endpointName string) string {
 	return name
 }
 
+func PortName(endpoint dw.Endpoint) string {
+	if len(endpoint.Name) <= 15 {
+		return endpoint.Name
+	}
+	return fmt.Sprintf("%d-%s", endpoint.TargetPort, endpoint.Protocol)
+}
+
 func ServiceName(workspaceId string) string {
 	return fmt.Sprintf("%s-%s", workspaceId, "service")
 }
@@ -95,3 +105,11 @@ func AutoMountSecretVolumeName(volumeName string) string {
 func AutoMountPVCVolumeName(pvcName string) string {
 	return fmt.Sprintf("automount-pvc-%s", pvcName)
 }
+
+func WorkspaceRoleName() string {
+	return "workspace"
+}
+
+func WorkspaceRolebindingName() string {
+	return constants.ServiceAccount + "dw"
+}
