@@ -1,5 +1,5 @@
 //
-// Copyright (c) 2019-2021 Red Hat, Inc.
+// Copyright (c) 2019-2022 Red Hat, Inc.
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
 // You may obtain a copy of the License at
@@ -47,29 +47,6 @@ func TestResolveDevWorkspaceKubernetesReference(t *testing.T) {
 	}
 }
 
-func TestResolveDevWorkspaceInternalRegistry(t *testing.T) {
-	tests := testutil.LoadAllTestsOrPanic(t, "testdata/internal-registry")
-	for _, tt := range tests {
-		t.Run(tt.Name, func(t *testing.T) {
-			// sanity check: input defines components
-			assert.True(t, len(tt.Input.DevWorkspace.Components) > 0, "Test case defines workspace with no components")
-			testResolverTools := getTestingTools(tt.Input, "test-ignored")
-
-			outputWorkspace, _, err := ResolveDevWorkspace(tt.Input.DevWorkspace, testResolverTools)
-			if tt.Output.ErrRegexp != nil && assert.Error(t, err) {
-				assert.Regexp(t, *tt.Output.ErrRegexp, err.Error(), "Error message should match")
-			} else {
-				if !assert.NoError(t, err, "Should not return error") {
-					return
-				}
-				assert.Truef(t, cmp.Equal(tt.Output.DevWorkspace, outputWorkspace, testutil.WorkspaceTemplateDiffOpts),
-					"DevWorkspace should match expected output:\n%s",
-					cmp.Diff(tt.Output.DevWorkspace, outputWorkspace, testutil.WorkspaceTemplateDiffOpts))
-			}
-		})
-	}
-}
-
 func TestResolveDevWorkspacePluginRegistry(t *testing.T) {
 	tests := testutil.LoadAllTestsOrPanic(t, "testdata/plugin-id")
 	for _, tt := range tests {
@@ -149,7 +126,6 @@ func TestResolveDevWorkspaceMissingDefaults(t *testing.T) {
 			// sanity check: input defines components
 			assert.True(t, len(tt.Input.DevWorkspace.Components) > 0, "Test case defines workspace with no components")
 			testResolverTools := getTestingTools(tt.Input, "")
-			testResolverTools.InternalRegistry = nil
 
 			outputWorkspace, _, err := ResolveDevWorkspace(tt.Input.DevWorkspace, testResolverTools)
 			if tt.Output.ErrRegexp != nil && assert.Error(t, err) {
@@ -212,29 +188,6 @@ func TestResolveDevWorkspaceTemplateNamespaceRestriction(t *testing.T) {
 	}
 }
 
-func TestResolveDevWorkspaceWorkspaceEnv(t *testing.T) {
-	tests := testutil.LoadAllTestsOrPanic(t, "testdata/workspace-env")
-	for _, tt := range tests {
-		t.Run(tt.Name, func(t *testing.T) {
-			// sanity check: input defines components
-			assert.True(t, len(tt.Input.DevWorkspace.Components) > 0, "Test case defines devworkspace with no components")
-			testResolverTools := getTestingTools(tt.Input, "test-ignored")
-
-			outputWorkspace, _, err := ResolveDevWorkspace(tt.Input.DevWorkspace, testResolverTools)
-			if tt.Output.ErrRegexp != nil && assert.Error(t, err) {
-				assert.Regexp(t, *tt.Output.ErrRegexp, err.Error(), "Error message should match")
-			} else {
-				if !assert.NoError(t, err, "Should not return error") {
-					return
-				}
-				assert.Truef(t, cmp.Equal(tt.Output.DevWorkspace, outputWorkspace, testutil.WorkspaceTemplateDiffOpts),
-					"DevWorkspace should match expected output:\n%s",
-					cmp.Diff(tt.Output.DevWorkspace, outputWorkspace, testutil.WorkspaceTemplateDiffOpts))
-			}
-		})
-	}
-}
-
 func TestMergesDuplicateVolumeComponents(t *testing.T) {
 	tests := testutil.LoadAllTestsOrPanic(t, "testdata/volume_merging")
 	for _, tt := range tests {
@@ -268,14 +221,8 @@ func getTestingTools(input testutil.TestInput, testNamespace string) ResolverToo
 		DevWorkspaceResources: input.DevWorkspaceResources,
 		Errors:                input.Errors,
 	}
-	testRegistry := &testutil.FakeInternalRegistry{
-		Plugins: input.DevWorkspaceResources,
-
-		Errors: input.Errors,
-	}
 	return ResolverTools{
 		Context:            context.Background(),
-		InternalRegistry:   testRegistry,
 		K8sClient:          testK8sClient,
 		HttpClient:         testHttpGetter,
 		WorkspaceNamespace: testNamespace,
