@@ -1,5 +1,5 @@
 //
-// Copyright (c) 2019-2021 Red Hat, Inc.
+// Copyright (c) 2019-2022 Red Hat, Inc.
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
 // You may obtain a copy of the License at
@@ -26,9 +26,12 @@ const (
 	asyncSidecarMemoryLimit   = "512Mi"
 	asyncServerMemoryRequest  = "256Mi"
 	asyncServerMemoryLimit    = "512Mi"
+
+	asyncStorageFinalizer = "controller.devfile.io/async-storage"
 )
 
 var asyncServerLabels = map[string]string{
-	"app.kubernetes.io/name":    "async-storage",
-	"app.kubernetes.io/part-of": "devworkspace-operator",
+	"app.kubernetes.io/name":                "async-storage",
+	"app.kubernetes.io/part-of":             "devworkspace-operator",
+	"controller.devfile.io/devworkspace_id": "",
 }
