@@ -1,5 +1,5 @@
 //
-// Copyright (c) 2019-2021 Red Hat, Inc.
+// Copyright (c) 2019-2022 Red Hat, Inc.
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
 // You may obtain a copy of the License at
@@ -19,23 +19,19 @@ import (
 	"context"
 	"errors"
 	"fmt"
-	"strings"
 
+	nsconfig "github.com/devfile/devworkspace-operator/pkg/provision/config"
 	"github.com/devfile/devworkspace-operator/pkg/provision/sync"
 	"k8s.io/apimachinery/pkg/fields"
 
+	dw "github.com/devfile/api/v2/pkg/apis/workspaces/v1alpha2"
 	"github.com/devfile/devworkspace-operator/apis/controller/v1alpha1"
 	maputils "github.com/devfile/devworkspace-operator/internal/map"
 	"github.com/devfile/devworkspace-operator/pkg/common"
 	"github.com/devfile/devworkspace-operator/pkg/config"
 	"github.com/devfile/devworkspace-operator/pkg/constants"
 	"github.com/devfile/devworkspace-operator/pkg/infrastructure"
-	"github.com/devfile/devworkspace-operator/pkg/provision/workspace/automount"
 
-	dw "github.com/devfile/api/v2/pkg/apis/workspaces/v1alpha2"
-
-	"github.com/google/go-cmp/cmp"
-	"github.com/google/go-cmp/cmp/cmpopts"
 	appsv1 "k8s.io/api/apps/v1"
 	corev1 "k8s.io/api/core/v1"
 	k8sErrors "k8s.io/apimachinery/pkg/api/errors"
@@ -62,24 +58,12 @@ var unrecoverablePodEventReasons = []string{
 	"ReplicaSetCreateError",
 }
 
-type DeploymentProvisioningStatus struct {
-	ProvisioningStatus
+var unrecoverableDeploymentConditionReasons = []string{
+	"FailedCreate",
 }
 
-var deploymentDiffOpts = cmp.Options{
-	cmpopts.IgnoreFields(appsv1.Deployment{}, "TypeMeta", "ObjectMeta", "Status"),
-	cmpopts.IgnoreFields(appsv1.DeploymentSpec{}, "RevisionHistoryLimit", "ProgressDeadlineSeconds"),
-	cmpopts.IgnoreFields(corev1.PodSpec{}, "DNSPolicy", "SchedulerName", "DeprecatedServiceAccount"),
-	cmpopts.IgnoreFields(corev1.Container{}, "TerminationMessagePath", "TerminationMessagePolicy", "ImagePullPolicy"),
-	cmpopts.SortSlices(func(a, b corev1.Container) bool {
-		return strings.Compare(a.Name, b.Name) > 0
-	}),
-	cmpopts.SortSlices(func(a, b corev1.Volume) bool {
-		return strings.Compare(a.Name, b.Name) > 0
-	}),
-	cmpopts.SortSlices(func(a, b corev1.VolumeMount) bool {
-		return strings.Compare(a.Name, b.Name) > 0
-	}),
+type DeploymentProvisioningStatus struct {
+	ProvisioningStatus
 }
 
 func SyncDeploymentToCluster(
@@ -88,34 +72,20 @@ func SyncDeploymentToCluster(
 	saName string,
 	clusterAPI sync.ClusterAPI) DeploymentProvisioningStatus {
 
-	automountPodAdditions, automountEnv, err := automount.GetAutoMountResources(clusterAPI, workspace.GetNamespace())
+	podTolerations, nodeSelector, err := nsconfig.GetNamespacePodTolerationsAndNodeSelector(workspace.Namespace, clusterAPI)
 	if err != nil {
-		var fatalErr *automount.FatalError
-		if errors.As(err, &fatalErr) {
-			return DeploymentProvisioningStatus{
-				ProvisioningStatus{Err: err, FailStartup: true},
-			}
-		} else {
-			return DeploymentProvisioningStatus{
-				ProvisioningStatus{Err: err},
-			}
-		}
-	}
-	if err := automount.CheckAutoMountVolumesForCollision(podAdditions, automountPodAdditions); err != nil {
 		return DeploymentProvisioningStatus{
-			ProvisioningStatus{Err: err, FailStartup: true},
+			ProvisioningStatus{
+				Message:     "failed to read pod tolerations and node selector from namespace",
+				Err:         err,
+				FailStartup: true,
+			},
 		}
 	}
-	podAdditions = append(podAdditions, automountPodAdditions...)
-
-	var envFromSourceAdditions []corev1.EnvFromSource
-	if automountEnv != nil {
-		envFromSourceAdditions = append(envFromSourceAdditions, automountEnv...)
-	}
 
 	// [design] we have to pass components and routing pod additions separately because we need mountsources from each
 	// component.
-	specDeployment, err := getSpecDeployment(workspace, podAdditions, envFromSourceAdditions, saName, clusterAPI.Scheme)
+	specDeployment, err := getSpecDeployment(workspace, podAdditions, saName, podTolerations, nodeSelector, clusterAPI.Scheme)
 	if err != nil {
 		return DeploymentProvisioningStatus{
 			ProvisioningStatus{
@@ -151,6 +121,16 @@ func SyncDeploymentToCluster(
 		}
 	}
 
+	deploymentHealthy, deploymentErrMsg := checkDeploymentConditions(clusterDeployment)
+	if !deploymentHealthy {
+		return DeploymentProvisioningStatus{
+			ProvisioningStatus: ProvisioningStatus{
+				FailStartup: true,
+				Message:     deploymentErrMsg,
+			},
+		}
+	}
+
 	failureMsg, checkErr := checkPodsState(workspace, clusterAPI)
 	if checkErr != nil {
 		return DeploymentProvisioningStatus{
@@ -189,9 +169,9 @@ func DeleteWorkspaceDeployment(ctx context.Context, workspace *dw.DevWorkspace,
 }
 
 // ScaleDeploymentToZero scales the cluster deployment to zero
-func ScaleDeploymentToZero(workspace *dw.DevWorkspace, client runtimeClient.Client) error {
+func ScaleDeploymentToZero(ctx context.Context, workspace *dw.DevWorkspace, client runtimeClient.Client) error {
 	patch := []byte(`{"spec":{"replicas": 0}}`)
-	err := client.Patch(context.Background(), &appsv1.Deployment{
+	err := client.Patch(ctx, &appsv1.Deployment{
 		ObjectMeta: metav1.ObjectMeta{
 			Namespace: workspace.Namespace,
 			Name:      common.DeploymentName(workspace.Status.DevWorkspaceId),
@@ -206,28 +186,34 @@ func ScaleDeploymentToZero(workspace *dw.DevWorkspace, client runtimeClient.Clie
 }
 
 func GetDevWorkspaceSecurityContext() *corev1.PodSecurityContext {
-	if !infrastructure.IsOpenShift() {
-		uID := int64(1234)
-		rootGID := int64(0)
-		nonRoot := true
-		return &corev1.PodSecurityContext{
-			RunAsUser:    &uID,
-			RunAsGroup:   &rootGID,
-			RunAsNonRoot: &nonRoot,
-		}
+	if infrastructure.IsOpenShift() {
+		return &corev1.PodSecurityContext{}
 	}
-	return &corev1.PodSecurityContext{}
+	return config.Workspace.PodSecurityContext
 }
 
 func checkDeploymentStatus(deployment *appsv1.Deployment) (ready bool) {
 	return deployment.Status.ReadyReplicas > 0
 }
 
+func checkDeploymentConditions(deployment *appsv1.Deployment) (healthy bool, errorMsg string) {
+	conditions := deployment.Status.Conditions
+	for _, condition := range conditions {
+		for _, unrecoverableReason := range unrecoverableDeploymentConditionReasons {
+			if condition.Reason == unrecoverableReason {
+				return false, fmt.Sprintf("Detected unrecoverable deployment condition: %s %s", condition.Reason, condition.Message)
+			}
+		}
+	}
+	return true, ""
+}
+
 func getSpecDeployment(
 	workspace *dw.DevWorkspace,
 	podAdditionsList []v1alpha1.PodAdditions,
-	envFromSourceAdditions []corev1.EnvFromSource,
 	saName string,
+	podTolerations []corev1.Toleration,
+	nodeSelector map[string]string,
 	scheme *runtime.Scheme) (*appsv1.Deployment, error) {
 	replicas := int32(1)
 	terminationGracePeriod := int64(10)
@@ -237,18 +223,11 @@ func getSpecDeployment(
 		return nil, err
 	}
 
-	creator := workspace.Labels[constants.DevWorkspaceCreatorLabel]
-	var envVars []corev1.EnvVar
-	envVars = append(envVars, CommonEnvironmentVariables(workspace.Name, workspace.Status.DevWorkspaceId, workspace.Namespace, creator)...)
 	for idx := range podAdditions.Containers {
-		podAdditions.Containers[idx].Env = append(podAdditions.Containers[idx].Env, envVars...)
 		podAdditions.Containers[idx].VolumeMounts = append(podAdditions.Containers[idx].VolumeMounts, podAdditions.VolumeMounts...)
-		podAdditions.Containers[idx].EnvFrom = append(podAdditions.Containers[idx].EnvFrom, envFromSourceAdditions...)
 	}
 	for idx := range podAdditions.InitContainers {
-		podAdditions.InitContainers[idx].Env = append(podAdditions.InitContainers[idx].Env, envVars...)
 		podAdditions.InitContainers[idx].VolumeMounts = append(podAdditions.InitContainers[idx].VolumeMounts, podAdditions.VolumeMounts...)
-		podAdditions.InitContainers[idx].EnvFrom = append(podAdditions.InitContainers[idx].EnvFrom, envFromSourceAdditions...)
 	}
 
 	deployment := &appsv1.Deployment{
@@ -294,6 +273,13 @@ func getSpecDeployment(
 		},
 	}
 
+	if podTolerations != nil && len(podTolerations) > 0 {
+		deployment.Spec.Template.Spec.Tolerations = podTolerations
+	}
+	if nodeSelector != nil && len(nodeSelector) > 0 {
+		deployment.Spec.Template.Spec.NodeSelector = nodeSelector
+	}
+
 	if needsPVCWorkaround(podAdditions) {
 		// Kubernetes creates directories in a PVC to support subpaths such that only the leaf directory has g+rwx permissions.
 		// This means that mounting the subpath e.g. <workspace-id>/plugins will result in the <workspace-id> directory being
