@@ -1,5 +1,5 @@
 //
-// Copyright (c) 2019-2021 Red Hat, Inc.
+// Copyright (c) 2019-2022 Red Hat, Inc.
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
 // You may obtain a copy of the License at
@@ -40,7 +40,7 @@ func (h *WebhookHandler) MutateWorkspaceV1alpha1OnCreate(_ context.Context, req
 	return h.returnPatched(req, wksp)
 }
 
-func (h *WebhookHandler) MutateWorkspaceV1alpha2OnCreate(_ context.Context, req admission.Request) admission.Response {
+func (h *WebhookHandler) MutateWorkspaceV1alpha2OnCreate(ctx context.Context, req admission.Request) admission.Response {
 	wksp := &dwv2.DevWorkspace{}
 	err := h.Decoder.Decode(req, wksp)
 	if err != nil {
@@ -49,6 +49,10 @@ func (h *WebhookHandler) MutateWorkspaceV1alpha2OnCreate(_ context.Context, req
 
 	wksp.Labels = maputils.Append(wksp.Labels, constants.DevWorkspaceCreatorLabel, req.UserInfo.UID)
 
+	if err := h.validateUserPermissions(ctx, req, wksp, nil); err != nil {
+		return admission.Denied(err.Error())
+	}
+
 	return h.returnPatched(req, wksp)
 }
 
@@ -85,7 +89,7 @@ func (h *WebhookHandler) MutateWorkspaceV1alpha1OnUpdate(_ context.Context, req
 	return admission.Allowed("new devworkspace has the same devworkspace creator as old one")
 }
 
-func (h *WebhookHandler) MutateWorkspaceV1alpha2OnUpdate(_ context.Context, req admission.Request) admission.Response {
+func (h *WebhookHandler) MutateWorkspaceV1alpha2OnUpdate(ctx context.Context, req admission.Request) admission.Response {
 	newWksp := &dwv2.DevWorkspace{}
 	oldWksp := &dwv2.DevWorkspace{}
 	err := h.parse(req, oldWksp, newWksp)
@@ -97,6 +101,10 @@ func (h *WebhookHandler) MutateWorkspaceV1alpha2OnUpdate(_ context.Context, req
 		return admission.Denied(msg)
 	}
 
+	if err := h.validateUserPermissions(ctx, req, newWksp, oldWksp); err != nil {
+		return admission.Denied(err.Error())
+	}
+
 	oldCreator, found := oldWksp.Labels[constants.DevWorkspaceCreatorLabel]
 	if !found {
 		return admission.Denied(fmt.Sprintf("label '%s' is missing. Please recreate devworkspace to get it initialized", constants.DevWorkspaceCreatorLabel))
